from typing import Annotated
from contextlib import asynccontextmanager
from importlib.metadata import entry_points
import logging

from fastapi import FastAPI, Depends, BackgroundTasks, HTTPException, status
from fastapi.responses import FileResponse, HTMLResponse
from hatchet_sdk import EmptyModel
from jwt import decode as decode_jwt

from knoc.settings import settings
from knoc.workflows import all_wfs
from knoc.workflows.utils import ImportReport
from knoc.db import get_session

logger = logging.getLogger("knoc.server")


@asynccontextmanager
async def lifespan(app: FastAPI):
    # Load the FastAPI app
    logger.debug("App is Starting Up!")
    yield
    logger.info(
        "App is Shutting Down Gracefully!",
        extra={"markup": True},
    )


app = FastAPI(lifespan=lifespan)


def get_plugins():
    for entry_point in entry_points().select(group="knoc.api"):
        try:
            api = entry_point.load()
        except Exception as err:
            logger.warning(
                f"Cannot load settings module {entry_point.name} - see below"
            )
            logger.exception(err)
            continue
        if not isinstance(api, FastAPI):
            logger.warning(f"Settings module {entry_point.name} is not a FastAPI")
            continue
        app.mount("/" + entry_point.name, api)
        logger.info(f"Mounted {entry_point.name}")


get_plugins()


@app.get("/")
async def root():
    hatchet_console_url = decode_jwt(
        settings.hatchet_client.token, options={"verify_signature": False}
    )["server_url"]
    return HTMLResponse(
        f'<a href="{hatchet_console_url}">Hatchet console</a>'
        "<hr>"
        '<a href="/docs">OpenAPI docs</a>'
    )


@app.get("/process")
async def list_workflows() -> list[str]:
    return list(all_wfs.keys())


# @app.get("/process/{name}")
# async def process_foo_task(name: str):
#    wf = all_wfs[name]
#    breakpoint()
#    result = await wf.aio_run(EmptyModel())
#    return result
