import logging
from typing import Type, Tuple
from pathlib import Path
from xdg import BaseDirectory
from importlib.metadata import entry_points

from pydantic import BaseModel, create_model
from pydantic_settings import (
    BaseSettings,
    PydanticBaseSettingsSource,
    SettingsConfigDict,
    YamlConfigSettingsSource,
)

logger = logging.getLogger("knoc.settings")

# from hatchet_sdk import ClientConfig as HatchetClientConfig
#
# Minimal Mock hatchet config as it is amazingly slow to load.
# The real instanciation, using the settings defined here,
# is done in hatchet_client.py


class HatchetClientTlsConfig(BaseSettings):
    strategy: str = "none"


class HatchetClientLogConfig(BaseSettings):
    level: int = 40


class HatchetClientConfig(BaseSettings):
    model_config = SettingsConfigDict(
        env_prefix="HATCHET_CLIENT_",
    )

    token: str = ""
    tls: HatchetClientTlsConfig = HatchetClientTlsConfig()
    log: HatchetClientLogConfig = HatchetClientLogConfig()


config_files = [
    Path(BaseDirectory.xdg_config_home) / "knoc" / "conf.yaml",
    Path(Path.cwd().root) / "etc" / "knoc" / f"conf.yml",
]


class HatchetSettings(BaseModel):
    token: str = ""
    tlsstrategy: str = "none"


class DBSettings(BaseModel):
    # url: str = "postgresql+psycopg://knoc:knoc@knoc-db:5432/knoc"
    username: str = "knoc"
    password: str = "knoc"
    host: str = "knoc-db"
    port: int = 5432
    database: str = "knoc"
    echo: bool = False


class KBaseSettings(BaseSettings):
    model_config = SettingsConfigDict(
        nested_model_default_partial_update=True,
        env_nested_delimiter="_",
        env_file=(".env", ".env.prod"),
        extra="allow",
    )

    @classmethod
    def settings_customise_sources(
        cls,
        settings_cls: Type[BaseSettings],
        init_settings: PydanticBaseSettingsSource,
        env_settings: PydanticBaseSettingsSource,
        dotenv_settings: PydanticBaseSettingsSource,
        file_secret_settings: PydanticBaseSettingsSource,
    ) -> Tuple[PydanticBaseSettingsSource, ...]:
        configs = [
            YamlConfigSettingsSource(settings_cls, yaml_file=cf) for cf in config_files
        ]
        return (
            env_settings,
            init_settings,
            dotenv_settings,
            file_secret_settings,
            *configs,
        )

    hatchet_client: HatchetClientConfig = HatchetClientConfig()
    hatchet_client_debug: bool = False
    db: DBSettings = DBSettings()

    data_path: Path = Path("data")


def get_plugins() -> dict[str, type[BaseModel]]:
    plugin_settings: dict[str, type[BaseModel]] = {}
    for entry_point in entry_points().select(group="knoc.settings"):
        try:
            SubSettings = entry_point.load()
        except Exception as err:
            logger.warning(
                f"Cannot load settings module {entry_point.name} - see below"
            )
            logger.exception(err)
            continue
        if not issubclass(SubSettings, BaseModel):
            logger.warning(f"Settings module {entry_point.name} is not a BaseModel")
            continue
        plugin_settings[entry_point.name] = SubSettings
    return plugin_settings


plugin_settings = get_plugins()
Settings = create_model(
    "Settings",
    **plugin_settings,  # type: ignore
    __base__=KBaseSettings,
    __module__="knoc.settings",
)


settings = Settings()
pass
